# Defining the base clocks in this system
create_clock -period 20.000 -name ext_clock_in [get_ports {clk}]
create_clock -period 100.000 -name JTAG_clk [get_ports {altera_reserved_tck}]


# Deriving the output clocks from the PLL settings
derive_pll_clocks


# Creating some short names for the PLL outputs extracted from the derive_pll_clocks command
set System_Clock inst|the_pll|the_pll|altpll_component|auto_generated|pll1|clk[0]
set Remote_Config_Clock inst|the_pll|the_pll|altpll_component|auto_generated|pll1|clk[1]


# Creating all clock uncertainty
derive_clock_uncertainty -add


# Cutting paths between the JTAG and all other ports
set_clock_groups -asynchronous -group {JTAG_clk}


# Cutting paths for the JTAG tdo, tdi, and tms pins (ports)
set_false_path -to [get_ports altera_reserved_tdo]
set_false_path -from [get_ports altera_reserved_tdi]
set_false_path -from [get_ports altera_reserved_tms]


# Cutting paths to/from the slow I/O
set_false_path -from [get_ports in_port_to_the_button[*]]
set_false_path -to [get_ports out_port_from_the_led[*]]


# Flash interface constraints.  The tri-state bridge operates at 60MHz (T=16.667ns) and the constraints contain
# margins of 8ns and -4ns.
set_max_delay -to [get_ports *cfi_flash*] 8
set_min_delay -to [get_ports *cfi_flash*] 0
set_max_delay -from [get_ports data_to_and_from_the_cfi_flash[*]] 4
set_min_delay -from [get_ports data_to_and_from_the_cfi_flash[*]] -4



# Cutting the input reset path since SOPC Builder will synchronize all resets
set_false_path -from [get_ports reset_n]


# There is an asyncronous clock crosser between the PLL component and Nios II.
# Cutting the paths between these domains since there are no other transfers between these two domains
set_false_path -from [get_clocks $System_Clock] -to [get_clocks $Remote_Config_Clock]
set_false_path -from [get_clocks $Remote_Config_Clock] -to [get_clocks $System_Clock]


# There is an asyncronous clock crosser between the PLL component and Nios II.
# Cutting the paths between these domains since there are no other transfers between these two domains
set_false_path -from [get_clocks ext_clock_in] -to [get_clocks $System_Clock]
set_false_path -from [get_clocks $System_Clock] -to [get_clocks ext_clock_in]